#!/bin/bash

# Provides generic functionality to deal with connectors
# Pre-requisites: $OUTSYSTEMS_HOME environment variable set

if [ -z "$OUTSYSTEMS_HOME" ]; then
        echo "OUTSYSTEMS_HOME environment variable not set."
        exit 1
fi

source $OUTSYSTEMS_HOME/common.sh

# Should come from common.sh, but we set it here just to be safe
if [ -z "$AVAILABLE_CONNECTORS_DIR" ]; then
        AVAILABLE_CONNECTORS_DIR="$OUTSYSTEMS_HOME/connectors"
fi

# Should come from common.sh, but we set it here just to be safe
if [ -z "$ACTIVE_CONNECTORS_DIR" ]; then
        ACTIVE_CONNECTORS_DIR="$OUTSYSTEMS_HOME/lib/active.connectors"
fi

# If you change anything in this array, ensure that you also change the indexes
declare -a REQUIRED_EXTENSION_POINTS=('prepareConnector' 'startConnectorRelatedServicesIfNeeded' 'stopConnectorRelatedServices');
FN_PREPARE_CONNECTOR_INDEX=0
FN_START_CONNECTOR_SERVICES=1
FN_STOP_CONNECTOR_SERVICES=2

declare -a ACTIVE_CONNECTORS
declare -a ACTIVE_EXTENSION_POINTS_NAMES

function createLink() {
        if [[ -z "$1" || -z "$2" || -z "$3" ]]
        then
                echo "ERROR: three parameters (existing location, target directory name, target file name) must be provided. Links not created."
                return -1
        fi
        
	local EXISTING_LOCATION="$1"
	local TARGET_DIR="$ACTIVE_CONNECTORS_DIR/$2"
	local LINK_LOCATION="$TARGET_DIR/$3"
	
	mkdir -p "$TARGET_DIR"
	
	connectors_debug "Creating link from $LINK_LOCATION to $EXISTING_LOCATION."	
	ln -fs "$EXISTING_LOCATION" "$LINK_LOCATION" >/dev/null 2>&1
}

#Creates a link from an original file, into a destination directory preserving its filename.
#It will only create the link if needed.
#Syntax: createLinkSimple path_To_SourceFilename destination_path  
function createLinkSimple(){
	LINK_FILENAME=`basename $1`
	LINK_FULLPATH="$2/$LINK_FILENAME"
	
	if [ ! -L "$LINK_FULLPATH" ]
	then
		connectors_debug "Creating link from $1 to $2/$LINK_FILENAME"	
		ln -fs "$1" "$LINK_FULLPATH" >/dev/null 2>&1
	fi	
}

# Changes the name of a ShellScript function in memory.
# Input: functionName newFunctionName
# Result: the function will now have the new name and the old name cannot be used to call it anymore
function rename_function() {
    local ORIGINAL_FUNCTION=$(declare -f "$1")
    local NEW_NAME_FUNCTION="$2${ORIGINAL_FUNCTION#$1}"
    eval "$NEW_NAME_FUNCTION"
}

# Checks if a function with a given name has been loaded into memory.
function fn_exists() {
  [ `type -t $1`"" == 'function' ]
}

CONNECTORS_DEBUG=""
function connectors_debug() {
        if [ ! -z "$CONNECTORS_DEBUG" ]; then
                local LOG_FILE="/opt/outsystems/platform/logs/ConfigurationTool-connectors.log"
                local LOG_DATE=`date`
                echo "$LOG_DATE [${FUNCNAME[1]}] $*" >>"$LOG_FILE"
        fi
}

function log_info() {
	if [ ! -z "$LOGFILENAME" ]; then
		echo "$(date +%F\ %T)     INFO  [${FUNCNAME[1]}] $*" >> "$LOGFILENAME"
	fi
}

function getPlainConnectorName() {
        echo "$1" | tr -d "."
}

function getFunctionNameFromConnectorAndExtensionPoint() {
        local CONNECTOR="$1"
        local EXTENSION_POINT_NAME="$2"

        connectors_debug "Retrieving function name for $EXTENSION_POINT_NAME in $CONNECTOR."
        local PLAIN_CONNECTOR_NAME=`getPlainConnectorName "$CONNECTOR"`

        local FUNCTION_NAME="${PLAIN_CONNECTOR_NAME}_${EXTENSION_POINT_NAME}"
        connectors_debug "Function name is $FUNCTION_NAME"

        echo "$FUNCTION_NAME"
}

function executeActiveExtensionPoint() {

        local CONNECTOR="$1"
        local EXTENSION_POINT_NAME="$2"
        local THE_FUNCTION=`getFunctionNameFromConnectorAndExtensionPoint "$1" "$2"`

        if fn_exists "$THE_FUNCTION"; then
		connectors_debug "Attempting to execute $THE_FUNCTION..."
		connectors_debug "`type $THE_FUNCTION`"
	
		$THE_FUNCTION
	
		connectors_debug "Executed $THE_FUNCTION."
	else
		connectors_debug "$EXTENSION_POINT_NAME function does not exist for conector $CONNECTOR. Not executed."	
	fi
}

function getConnectorSymlinkBaseDir() {
	local CONNECTOR="$1"
	echo `unzip -q -c $AVAILABLE_CONNECTORS_DIR/$CONNECTOR META-INF/MANIFEST.MF | grep "Class-Path" | cut -d ":" -f 2 | cut -d " " -f 2 | cut -d "/" -f 1`
}

function activateConnector() {

        local CONNECTOR="$1"
        local EMPTY_CONNECTOR=""

	if [[ ! -z "$2" && "$2" == "EMPTY" ]]; then
		EMPTY_CONNECTOR="1";
        fi

        connectors_debug "Creating symlink for $CONNECTOR in $ACTIVE_CONNECTORS_DIR."
        ln -f "$AVAILABLE_CONNECTORS_DIR/$CONNECTOR" "$ACTIVE_CONNECTORS_DIR/$CONNECTOR"

        local CONNECTOR_NAME=`getPlainConnectorName "$CONNECTOR"`

        connectors_debug "Active connectors before adding current one: ${ACTIVE_CONNECTORS[@]}"
        ACTIVE_CONNECTORS=("${ACTIVE_CONNECTORS[@]}" "$CONNECTOR_NAME")

        connectors_debug "Active connectors after adding current one: ${ACTIVE_CONNECTORS[@]}"
        
        # We only need to go through the extension points to rename them if the conector has them in the .SH 
        if [ -z "$EMPTY_CONNECTOR" ]; then
        	
			for EP in "${REQUIRED_EXTENSION_POINTS[@]}"
			do
				connectors_debug "Loading extension point $EP."
		
				local FUNCTION_NAME=`getFunctionNameFromConnectorAndExtensionPoint "$CONNECTOR" "$EP"`
		
				# We need to rename the function (i.e. pefix it with the conector name) so that we can have all extension points in memory
				# Extension points functions have the same name, as per their definition, in all connectors
				rename_function $EP $FUNCTION_NAME
			done
        fi

		callPrepareConnector $CONNECTOR
        
        connectors_debug "Finished activating $CONNECTOR."
}

function callPrepareConnector() {
        executeActiveExtensionPoint "$1" "${REQUIRED_EXTENSION_POINTS[$FN_PREPARE_CONNECTOR_INDEX]}"
}

function callStartConnectorRelatedServicesIfNeeded() {
        executeActiveExtensionPoint "$1" "${REQUIRED_EXTENSION_POINTS[$FN_START_CONNECTOR_SERVICES]}"
}

function activeConnectorsStartConnectorRelatedServicesIfNeeded() {
        connectors_debug "Active connectors: ${ACTIVE_CONNECTORS[@]}"
        for CONNECTOR in ${ACTIVE_CONNECTORS[@]}
        do
                callStartConnectorRelatedServicesIfNeeded $CONNECTOR
        done
}

function callStopConnectorRelatedServices() {
        executeActiveExtensionPoint "$1" "${REQUIRED_EXTENSION_POINTS[$FN_STOP_CONNECTOR_SERVICES]}"
}

function activeConnectorsStopConnectorRelatedServices() {
        connectors_debug "Active connectors: ${ACTIVE_CONNECTORS[@]}"
        for CONNECTOR in ${ACTIVE_CONNECTORS[@]}
        do
                callStopConnectorRelatedServices $CONNECTOR
        done
}

function processAvailableConnectors() {

        # avoid Control+C during connectors re-creation
        trap '' 2

        # Cleanup active.connectors before activating any of the connectors (clean state)
        rm -rf "$ACTIVE_CONNECTORS_DIR"
        mkdir "$ACTIVE_CONNECTORS_DIR"

        local CONNECTOR_SH_FILE="connector.sh"

        cd $AVAILABLE_CONNECTORS_DIR
        AVAILABLE_CONNECTORS=*.jar

        for CONNECTOR in $AVAILABLE_CONNECTORS
        do
                connectors_debug "Processing $CONNECTOR connector..."
                local TMPDIR=$(mktemp -d "/tmp/$CONNECTOR.XXXXXXXXXXX")
                unzip -q "$CONNECTOR" "$CONNECTOR_SH_FILE" -d "$TMPDIR" >/dev/null 2>&1

                if [ -f "$TMPDIR/$CONNECTOR_SH_FILE" ]; then
                        source "$TMPDIR/$CONNECTOR_SH_FILE"

                        connectors_debug "Connector $CONNECTOR will be active."
                        activateConnector "$CONNECTOR"
                else
                        connectors_debug "$CONNECTOR does not have $CONNECTOR_SH_FILE file. Connector will be active with empty extension points."
                        activateConnector "$CONNECTOR" "EMPTY"
                fi
                rm -rf $TMPDIR
        done
        cd - >/dev/null 2>&1

        # Change owner of connector files to avoid access problems
        chown -R outsystems:outsystems "$ACTIVE_CONNECTORS_DIR"
		
		# allow Control+C
		trap 2
}
